// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.casetool.together;

import java.io.File;
import java.util.Iterator;

import de.uka.ilkd.key.casetool.HashMapOfClassifier;
import de.uka.ilkd.key.casetool.UMLOCLAssociation;
import de.uka.ilkd.key.casetool.UMLOCLClassifier;
import de.uka.ilkd.key.casetool.UMLOCLJavaModel;

import com.togethersoft.openapi.rwi.RwiModel;
import com.togethersoft.openapi.rwi.RwiNode;
import com.togethersoft.openapi.rwi.RwiDiagram;
import com.togethersoft.openapi.rwi.RwiElement;
import com.togethersoft.openapi.rwi.RwiModelAccess;
import com.togethersoft.openapi.rwi.RwiPackage;
import com.togethersoft.openapi.rwi.RwiProperty;
import com.togethersoft.openapi.rwi.enum.RwiPackageEnumeration;
import com.togethersoft.openapi.rwi.enum.RwiNodeEnumeration;
import com.togethersoft.openapi.sci.SciModel;
import com.togethersoft.openapi.sci.SciModelAccess;
import com.togethersoft.openapi.ide.project.*;
import com.togethersoft.openapi.ide.IdeAccess;

public class UMLOCLTogetherModel extends UMLOCLJavaModel {
    //debug flag
    private static final boolean DEBUG = false;
    private static String pathSep = File.separator;
    private static final String TOGETHER_UNIQUE_NAME_PREFIX = "<oiref:java#Class#";
    private static final String TOGETHER_UNIQUE_NAME_POSTFIX = ":oiref>";


    public UMLOCLTogetherModel() {
	super(getTogetherProjectDir());
    }

    public static String getTogetherProjectDir() {
	IdeProject prj = IdeProjectManagerAccess
	    .getProjectManager().getActiveProject();
	if (prj != null) {
	    String tprFile = prj.getFileName();
	    return  tprFile.substring(0, tprFile.lastIndexOf(pathSep));
	} else
	    return "";
    }

    
    public HashMapOfClassifier getUMLOCLClassifiers() {
	      
	super.getUMLOCLClassifiers();
	// find associations
	RwiModel rwiModel = RwiModelAccess.getModel();		
	RwiPackageEnumeration rwiRoots=rwiModel.
	    rootPackages(RwiProperty.MODEL);
	while (rwiRoots.hasMoreElements()) {
	    RwiPackage rwiPackage = rwiRoots.nextRwiPackage();
	    processAssociations(rwiPackage);
	}

	if (DEBUG)
	    System.out.println("------------- found the following "+
			       "associations ------------------");

	Iterator cit = classifiers.values().iterator();
	UMLOCLClassifier c;
	while (cit.hasNext()) {
	    c = (UMLOCLClassifier)cit.next();
	    Iterator it = c.getAssociations().values().iterator();
	    UMLOCLAssociation assoc;
	    while (it.hasNext()) {
		assoc = (UMLOCLAssociation)it.next();
		if (DEBUG)
		    System.out.println("assoc from: "
				       +assoc.getSource().getName()+
				       " to: "+assoc.getTarget().getName());
	    }
	}
	return classifiers;
    }

    protected void processAssociations(RwiPackage rwiPackage){
	SciModel sciModel = SciModelAccess.getModel();
	RwiModel rwiModel = RwiModelAccess.getModel();

	RwiNodeEnumeration rwiNodesEnum = rwiPackage.nodes();
	RwiNode rwiNode;
	TogetherReprModelClass cls;
	while (rwiNodesEnum.hasMoreElements()){
	    rwiNode = rwiNodesEnum.nextRwiNode();
	    cls = new TogetherReprModelClass(rwiNode, rwiModel, null);
	    cls.getAssociations(classifiers);
	}	
	RwiPackageEnumeration subpackages = rwiPackage.subpackages();
	while (subpackages.hasMoreElements()) {
	    RwiPackage subpackage = subpackages.nextRwiPackage();
	    processAssociations(subpackage);
	}
    }

    /** Given the full name (i.e. including package prefix, e.g. "java.lang.Object") 
     *  of a classifier contained in the current model, this method returns the
     *  corresponding TogetherReprModelClass.
     * @param fullName the name, including package prefix, of a classifier
     * in the current UML model.
     * @return the corresponding TogetherReprModelClass
     */
    public TogetherReprModelClass getTogetherReprModelClass(String fullName) {
	String uniqueName = TOGETHER_UNIQUE_NAME_PREFIX + fullName 
	    + TOGETHER_UNIQUE_NAME_POSTFIX;
	RwiModel rwiModel = RwiModelAccess.getModel();
	RwiElement elem = rwiModel.findElement(uniqueName);
	TogetherReprModelClass togetherClass = null;
	if (elem != null && elem instanceof RwiNode) {
	    RwiNode rwiNode = (RwiNode)elem;
	    RwiDiagram diagram = rwiNode.getContainingPackage().getPhysicalDiagram();
	    togetherClass
		= new TogetherReprModelClass(rwiNode, rwiModel, diagram);
	}
	return togetherClass;
    }
}
