// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.gui;

import java.util.Properties;
import java.util.LinkedList;
import java.util.Iterator;

/** This class encapsulates the information which 
 *  decision procedure should be used.
 */
public class DecisionProcedureSettings implements Settings {


    private static final String DECISION_PROCEDURE = "[DecisionProcedure]";
    public static final String SIMPLIFY = "SIMPLIFY";
    public static final String ICS = "ICS";

    private String decision_procedure = SIMPLIFY;

    private LinkedList listenerList = new LinkedList();

    // getter
    public String getDecisionProcedure() {
	return decision_procedure;
    }

    // setter
    public void setDecisionProcedure(String s) {
        if(!s.equals(decision_procedure)) {
  	  decision_procedure = s;
	  fireSettingsChanged();
	}
    }

    public boolean useSimplify() {
	return decision_procedure.equals(SIMPLIFY);
    }

    public boolean useICS() {
	return decision_procedure.equals(ICS);
    }


    /** gets a Properties object and has to perform the necessary
     * steps in order to change this object in a way that it
     * represents the stored settings
     */
    public void readSettings(Properties props) {
	String dec_proc_string = 
	    props.getProperty(DECISION_PROCEDURE);

	if (dec_proc_string==null ||
	    dec_proc_string.equals(SIMPLIFY)) 
	    decision_procedure = SIMPLIFY;
	else if (dec_proc_string.equals(ICS))
	    decision_procedure = ICS;
	else 
	    decision_procedure = SIMPLIFY;
    }


    /** implements the method required by the Settings interface. The
     * settings are written to the given Properties object. Only entries of the form 
     * <key> = <value> (,<value>)* are allowed.
     * @param props the Properties object where to write the settings as (key, value) pair
     */
    public void writeSettings(Properties props) {
	props.setProperty(DECISION_PROCEDURE, getDecisionProcedure());
    }

    /** sends the message that the state of this setting has been
     * changed to its registered listeners (not thread-safe)
     */
    protected void fireSettingsChanged() {
	Iterator it = listenerList.iterator();
	while (it.hasNext()) {	    
	    ((SettingsListener)it.next()).settingsChanged(new GUIEvent(this));
	}
    }

    /** adds a listener to the settings object 
     * @param l the listener
     */
    public void addSettingsListener(SettingsListener l) {
	listenerList.add(l);
    }

}
