// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.gui;
/** this class implements a TreeModel that can be displayed using the
 * JTree class framework 
 */
import javax.swing.tree.*;
import de.uka.ilkd.key.proof.Node;

class GUIBranchNode extends GUIAbstractTreeNode 
                    implements TreeNode {

    private Node   subTree;
    private Object label;
    
    public GUIBranchNode(GUIProofTreeModel tree,
			 Node subTree,
			 Object   label) {
	super ( tree );
	this.subTree = subTree;
	this.label = label;
	for (int i=0; i<getChildCount(); i++) {
	    getChildAt(i);
	}
    }

    
    public TreeNode getChildAt(int childIndex) {
	int count = 0;
	Node n = subTree;
	while ( childIndex != count 
	        && n.childrenCount() == 1 ) {
	    count++;
	    n = n.child(0);
	}
	if ( childIndex == count ) {
	    return getProofTreeModel ().getProofTreeNode(n);
	} else {
	    return findBranch ( n.child(childIndex-count-1) );
	}
    }

    public int getChildCount() {
	int count = 1;
	int ncc = 0;
	Node n = subTree;
	while ( (ncc = n.childrenCount()) == 1 ) {
	    count++;
	    n = n.child(0);
	}
	return count + ncc;
    }
    
    
    public TreeNode getParent() {
	Node n = subTree.parent();
	if ( n==null ) {
	    return null;
	} else {
	    while (n.parent()!=null
		   && n.parent().childrenCount()==1) {
		n = n.parent();
	    }
	    return findBranch(n);
	}
    }

    public Node getNode() {
	return subTree;
    }

    // signalled by GUIProofTreeModel when the user has altered the value
    public void setLabel(String s) {
	subTree.setBranchLabel(s);
    }

    public boolean isLeaf() {
	return false;
    }

    public String toString() {
	String res = subTree.getBranchLabel ();
	if ( res == null )
	    return label.toString();
	return res;
    }


}
