// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.java.annotation;

import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.ArrayOfTerm;

public class LoopInvariantAnnotation extends TermContainerAnnotation{

    private final Term invariant;
    private final Term variant;
    private final ArrayOfTerm assignable;
    // array containing terms at indices 2n and program variables for
    // storing the initial values of these terms at indices 2n+1 
    // (for 0<=n<olds.size()/2). 
    private final ArrayOfTerm olds;

    public LoopInvariantAnnotation(Term invariant, ArrayOfTerm assignable, 
				   ArrayOfTerm olds, Term variant){
	this.invariant = invariant;
	this.variant = variant;
	this.assignable = assignable;
	this.olds = olds;
    }    

    public LoopInvariantAnnotation(Term invariant, ArrayOfTerm assignable){
	this(invariant, assignable, new ArrayOfTerm(), null);
    } 

    public int getTermCount(){
	int count = assignable.size();
	count += olds.size();
	if(invariant != null) count++;
	if(variant != null) count++;
	return count;
    }

    public ArrayOfTerm getArrayOfTerm(){
	Term[] ta = new Term[getTermCount()];
	int i = 0;
	if(invariant != null){
	    ta[i++] = invariant;
	}
	if(variant != null){
	    ta[i++] = variant;
	}
	for(int j = 0; j<assignable.size(); j++){
	    ta[i++] = assignable.getTerm(j);
	}
	for(int j = 0; j<olds.size(); j++){
	    ta[i++] = olds.getTerm(j);
	}
	return new ArrayOfTerm(ta);
    }

    public TermContainerAnnotation create(TermContainerAnnotation copy, 
					  ArrayOfTerm aot){
	LoopInvariantAnnotation tca = (LoopInvariantAnnotation) copy;
	Term inv = null;
	Term var = null;
	int i = 0;
	if(tca.invariant() != null){
	    inv = aot.getTerm(i++);
	}
	if(tca.variant() != null){
	    var = aot.getTerm(i++);
	}
	Term[] assign = new Term[tca.assignable().size()];
	for(int j = 0; j < assign.length; j++){
	    assign[j] = aot.getTerm(j+i);
	}
	Term[] old = new Term[tca.olds().size()];
	for(int j = 0; j < old.length; j++){
	    old[j] = aot.getTerm(j+i+assign.length);
	}
	return new LoopInvariantAnnotation(inv, new ArrayOfTerm(assign), 
					   new ArrayOfTerm(old), var);
    }

    public Term invariant(){
	return invariant;
    }

    public Term variant(){
	return variant;
    }

    public ArrayOfTerm assignable(){
	return assignable;
    }

    public ArrayOfTerm olds(){
	return olds;
    }

    public String toString(){
	String result = "invariant: "+invariant+"\nvariant: "+variant+
	    "\nassignable: ";
	for(int i=0; i<assignable.size(); i++){
	    result += " "+assignable.getTerm(i);
	}
	result += "\nold: "+olds;
	for(int i=0; i<olds.size(); i++){
	    result += assignable.getTerm(i)+(i%2==0? "=": " ");
	}
	return result;
    }

}
