// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.jml;

import java.util.*;

import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.abstraction.*;
import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.java.declaration.modifier.Public;
import de.uka.ilkd.key.java.expression.literal.BooleanLiteral;
import de.uka.ilkd.key.java.expression.operator.CopyAssignment;
import de.uka.ilkd.key.java.expression.operator.New;
import de.uka.ilkd.key.java.recoderext.ClassInitializeMethodBuilder;
import de.uka.ilkd.key.java.recoderext.ImplicitFieldAdder;
import de.uka.ilkd.key.java.reference.ReferencePrefix;
import de.uka.ilkd.key.java.reference.ExecutionContext;
import de.uka.ilkd.key.java.reference.TypeRef;
import de.uka.ilkd.key.java.reference.TypeReference;
import de.uka.ilkd.key.java.statement.MethodFrame;
import de.uka.ilkd.key.java.statement.CatchAllStatement;
import de.uka.ilkd.key.java.statement.MethodBodyStatement;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.logic.sort.ObjectSort;
import de.uka.ilkd.key.logic.sort.SortDefiningSymbols;
import de.uka.ilkd.key.proof.mgt.JavaModelMethod;
import de.uka.ilkd.key.proof.mgt.ListOfQuantifierPrefixEntry;
import de.uka.ilkd.key.proof.mgt.QuantifierPrefixEntry;
import de.uka.ilkd.key.rule.UpdateSimplifier;

/** 
 * A generic jml methodspecification used for encapsulating lightweight
 * and behavioral methodspecifications
 */
public class JMLMethodSpec extends JMLSpec implements JMLLemmaMethodSpec, 
						      AssignableSpec{

    private static final UpdateSimplifier simplifier = new UpdateSimplifier();

    private static final java.util.Comparator comparator = new java.util.Comparator() {
        public int compare(Object o1, Object o2) {
            return (""+o1).compareTo((""+o2).toString());
        }
    };

    
    protected static int excCondCount = 0;
    protected static int resultVarCount = 0;
    protected static int logicVarCount = 0;
    protected static int exceptionVarCount = 0;

    // counter for distinguishing different specs of one method.
    protected int id;
    // contains an assignable keyword (like nothing, everithing, ...), 
    // if one occurd in the specification.
    protected String assignableMode = null;
    protected ProgramVariable resultVar = null;
    protected TermFactory tf;
    //caches the classdeclaration from cSpec
    protected TypeDeclaration cd;
    protected JavaInfo ji;
    protected Services services;
    private Term pre;
    private Term post;
    protected TreeMap lv2old;
    protected Term diverges;
    protected Namespace params;
    protected StatementBlock mBS = null;
    protected static Term trueLitTerm=null;
    protected static Term falseLitTerm=null;
    protected HashMap term2old;
    protected SetOfTerm assignableVariables = SetAsListOfTerm.EMPTY_SET;
    protected JMLClassSpec cSpec;
    protected SetOfSignals signals = SetAsListOfSignals.EMPTY_SET;
    protected static Term trueTerm = null;
    protected static Term falseTerm = null;
    protected static Term nullTerm = null;
    protected NamespaceSet nss;
    protected ProgramMethod pm;
    protected String inhFrom = "";
    protected ProgramVariable excVar;
    protected KeYJavaType exc;
    private Term pi1 = null;
    private ListOfQuantifierPrefixEntry oldLVs = null;
    private CatchAllStatement catchAllStmt;
    //contains the self-variable used in an inherited methodspec.
    protected ReferencePrefix inheritedPrefix = null;
    protected Term self; 
    protected HashMap modalityTermForEnsuresCache = new HashMap();
    private String javaPath;
    
    //flag which is set to false iff static initialisation features should be suppressed
    protected boolean staticInit=false;


    public JMLMethodSpec(ProgramMethod pm, TermFactory tf, 
			 Services services, Namespace params, 
			 HashMap term2old, JMLClassSpec cSpec, 
			 NamespaceSet nss, String javaPath){
	this.pm = pm;
	progVarNS = new Namespace(cSpec.getProgramVariableNS());
	funcNS = new Namespace(cSpec.getFunctionNS());
	this.cd = cSpec.getClassDeclaration();
	this.nss = nss;
	this.tf = tf;
	this.ji = services.getJavaInfo();
	this.services = services;
	this.javaPath = javaPath;
	pre = null;
	post = null;
	if(nullTerm == null){
	    nullTerm = tf.createFunctionTerm(Op.NULL);
	}
	if(trueLitTerm == null){
	    trueLitTerm = services.getTypeConverter()
		.convertToLogicElement(BooleanLiteral.TRUE);
	}
	if(falseLitTerm == null){
	    falseLitTerm = services.getTypeConverter()
		.convertToLogicElement(BooleanLiteral.FALSE);
	}
	if(trueTerm == null){
	    trueTerm = tf.createJunctorTerm(Op.TRUE);
	}
	if(falseTerm == null){
	    falseTerm = tf.createJunctorTerm(Op.FALSE);
	}
	if(!pm.isStatic()){
	    self = tf.createVariableTerm((ProgramVariable) 
					 cSpec.getInstancePrefix());
	    //adds self!=null && self.<created> == true or 
	    //self.<classInitialized> == true to the precondition
	    if(!(pm.getMethodDeclaration() instanceof Constructor)){
		Term t = tf.createEqualityTerm(self,nullTerm);
		addPre(tf.createJunctorTerm(Op.NOT, new Term[]{t}));
		ProgramVariable created = services.getJavaInfo().
		    getAttribute(ImplicitFieldAdder.IMPLICIT_CREATED, 
				 services.getJavaInfo().getJavaLangObject());
		Term eqc = tf.createAttributeTerm(created, self);
		addPre(tf.createEqualityTerm(eqc, trueLitTerm));
	    }
	}        
	if(staticInit && !pm.getName().equals
	        (ClassInitializeMethodBuilder.CLASS_INITIALIZE_IDENTIFIER)){
	    ProgramVariable ci = 
	        services.getJavaInfo().getAttribute
	        (ImplicitFieldAdder.IMPLICIT_CLASS_INITIALIZED, 
	                services.getJavaInfo().getKeYJavaType(cd));
	    addPre(tf.createEqualityTerm(tf.createVariableTerm(ci), 
					 trueLitTerm));
	}

	services.getImplementation2SpecMap().addMethodSpec(this);
	diverges = null;
	this.params = params;
	this.term2old = term2old;
	this.cSpec = cSpec;
	exc = services.getJavaInfo().getTypeByClassName("Exception");
	excVar = new ProgramVariable(new ProgramElementName(
			       "_exc"+(exceptionVarCount++)), exc);
	progVarNS.add(excVar);
	resultVar = makeResultVar();
    }

    protected JMLMethodSpec(){}

    public void setId(int id){
	this.id = id;
    }

    public int id(){
	return id;
    }

    /**
     * This is used for modelling specification inheritance for overwritten
     * methods.
     */
    public JMLMethodSpec cloneFor(ProgramMethod pm, JMLClassSpec cSpec){
	if(!isCloneableFor(pm)) return null;
	JMLMethodSpec cloned = new JMLMethodSpec();
	return cloneForHelper(cloned, pm, cSpec);
    }

    public JMLMethodSpec copy(){
	JMLMethodSpec copy = 
	    new JMLMethodSpec(pm, tf, services, params, 
			      term2old, cSpec, nss, javaPath);
	return copyHelper(copy);
    }

    public String toString(){
	return isValid() ?
	    inhFrom+"behavior speccase "+id+" for method "+pm.getName()+
	    "\nin context "+cd.getName()+
	    "\nrequires: "+(getPre()==null ? "true" : getPre().toString())
	    :
	    "Invalid method spec due to unsupported expression: "+
	    nsEx.expression();
    } 

    public ProgramElement getProofStatement(){
	return createModalityTermForEnsures(post == null ? trueTerm : post, 
					    false, false, false).
	    javaBlock().program();
    } 


    /**
     * Checks if name, signature, accessibility and so on are equal for 
     * this.pm and pm. If this.pm and pm occur on the same branch in the
     * class hierarchy must be checked somewhere else!
     */
    public boolean isCloneableFor(ProgramMethod pm){
	if(!pm.getName().equals(this.pm.getName())){
	    return false;
	}
	if((pm.getTypeReference() != null && 
	    !pm.getTypeReference().equals(this.pm.getTypeReference())) ||
	   (pm.getTypeReference() == null && 
	    this.pm.getTypeReference() != null)){
	    return false;
	}
	if((pm.getThrown() != null && 
	    !pm.getThrown().equals(this.pm.getThrown())) ||
	    (pm.getThrown() == null && this.pm.getThrown() != null)){
	    return false;
	}
	if(pm.getParameterDeclarationCount() != 
	       this.pm.getParameterDeclarationCount()){
	    return false;
	}
	for(int i = 0; i<pm.getParameterDeclarationCount(); i++){
	    if(!pm.getParameterDeclarationAt(i).getTypeReference().equals(
		   this.pm.getParameterDeclarationAt(i).getTypeReference())){
		return false;
	    }
	}
	if(pm.getModifiers().size() != this.pm.getModifiers().size() && 
	   !(cd instanceof InterfaceDeclaration)){
	    return false;
	}
	Public p = new Public();
	for(int i = 0; i<pm.getModifiers().size(); i++){
	    boolean equal = false;
	    if((cd instanceof InterfaceDeclaration) && 
	       pm.getModifiers().getModifier(i).equals(p)){
		continue;
	    }
	    for(int j = 0; j<this.pm.getModifiers().size(); j++){
		if(pm.getModifiers().getModifier(i).equals(
		       this.pm.getModifiers().getModifier(j))){
		    equal = true;
		    break;
		}
	    }
	    if(!equal) return false;
	}
	return true;
    }

    /**
     * copys several fields to the incomplete copy <code>copy</copy> of this.
     */
    protected JMLMethodSpec copyHelper(JMLMethodSpec copy){
	copy.services = services;
	copy.progVarNS = progVarNS.copy();
	copy.funcNS = funcNS.copy();
	copy.tf = tf;
	copy.nss = nss;
	copy.params = params;
	copy.pre = pre;
	copy.post = post;
	copy.resultVar = resultVar;
	copy.assignableMode = getAssignableMode();
	copy.diverges = diverges;
	copy.term2old = (HashMap) term2old.clone();
	copy.assignableVariables = assignableVariables;
	copy.signals = signals;
	copy.excVar = excVar;
	copy.exc = exc;
	copy.ji = services.getJavaInfo();
	return copy;
    }

    protected JMLMethodSpec cloneForHelper(JMLMethodSpec clone, 
					   ProgramMethod pm, 
					   JMLClassSpec cSpec){
	clone = (JMLMethodSpec) copyHelper(clone);
	clone.id = id;
	clone.inhFrom = inhFrom.equals("") ?
	    "inherited from "+cd.getName()+" " : inhFrom;
	clone.cSpec = cSpec;
	//set new prefix
	if(!pm.isStatic()){
	    if(inheritedPrefix == null){
		clone.inheritedPrefix = getPrefix();
	    }else{
		clone.inheritedPrefix = inheritedPrefix; 
	    }
	    clone.self = tf.createVariableTerm((ProgramVariable) 
					       cSpec.getInstancePrefix());
	}
	clone.cd = cSpec.getClassDeclaration();
	KeYJavaType returnType;
	if(pm.getTypeReference() == null){
	    returnType = ji.getKeYJavaType("void");
	}else{
	    returnType = pm.getTypeReference().getKeYJavaType();
	}

	//replace the old arguments by the new ones
	Namespace newPNS = UsefulTools.buildParamNamespace(pm.getMethodDeclaration());
	Namespace oldPNS = clone.params;
	ArrayOfParameterDeclaration oldParams = this.pm.getParameters();
	ArrayOfParameterDeclaration newParams = pm.getParameters();
	clone.progVarNS.add(oldPNS.allElements());
	clone.params = newPNS;
	clone.post = updateParameters(oldParams, newParams, clone.post);
	clone.pre = updateParameters(oldParams, newParams, clone.pre);
        clone.pm = 
            new ProgramMethod(pm.getMethodDeclaration(), 
                        ji.getKeYJavaType(cSpec.getClassDeclaration()), 
                              returnType, PositionInfo.UNDEFINED);        
	SetOfSignals si = SetAsListOfSignals.EMPTY_SET;
	IteratorOfSignals it = clone.signals.iterator();
	while(it.hasNext()){
	    Signals s = it.next();
	    si = si.add(new Signals(s.type(), s.variable(), 
				    s.condition() == null? null :
				    clone.updatePrefix(
					 updateParameters(oldParams, newParams,
							  s.condition()))));
	}
	clone.signals = si;
	Iterator kit = term2old.keySet().iterator();
	clone.term2old = new HashMap();
	while(kit.hasNext()){
	    Term t = (Term) kit.next();
	    clone.term2old.put(clone.updatePrefix(
				    updateParameters(oldParams, newParams, t)),
				    term2old.get(t));
	}

	clone.progVarNS.add(cSpec.getProgramVariableNS());
	clone.funcNS.add(cSpec.getFunctionNS());

	if(staticInit && !pm.getName().equals
	        (ClassInitializeMethodBuilder.CLASS_INITIALIZE_IDENTIFIER) &&
	    !(cSpec.getClassDeclaration() instanceof InterfaceDeclaration)){
	    ProgramVariable ci = 
	        services.getJavaInfo().getAttribute
	        (ImplicitFieldAdder.IMPLICIT_CLASS_INITIALIZED, 
	                ji.getKeYJavaType(cSpec.getClassDeclaration()));
	    if(ci != null){
		clone.addPre(tf.createEqualityTerm(tf.createVariableTerm(ci), 
						   trueLitTerm));
	    }
	}
	clone.updateAssignableLocations();
        clone.javaPath = javaPath;
	return clone;
    }

    /**
     * Replaces the inherited prefix and the inherited parameters
     * with the current prefix and the current parameters in all assignable
     * terms.
     */
    private void updateAssignableLocations(){
	if(!(pm.isStatic() || 
	     pm.getMethodDeclaration() instanceof Constructor)){
	    SetOfTerm old = assignableVariables;
	    assignableVariables = SetAsListOfTerm.EMPTY_SET;
	    IteratorOfTerm it = old.iterator();
	    while(it.hasNext()){
		assignableVariables = 
		    assignableVariables.add(replaceSelf(it.next()));
	    }
	}
    }

    public ReferencePrefix getInheritedPrefix(){
	return inheritedPrefix;
    }

    public void clearModalityTermForEnsuresCache(){
	modalityTermForEnsuresCache.clear();
    }

    /**
     * Replaces occurences of the inherited prefix and the inherited parameters
     * in <code>t</code> with the current prefix and the current parameters,
     * respectively.
     */
    private Term replaceSelf(Term t){
	if(t.op() == inheritedPrefix){
	    return self;
	}else if(t.op() instanceof ProgramVariable){
	    if(params.lookup(t.op().name()) != null){
		return tf.createVariableTerm((ProgramVariable)
					     params.lookup(t.op().name()));
	    }
	    return t;
	}else{
	    Term[] subs = new Term[t.arity()];
	    for(int i = 0; i<t.arity(); i++){
		subs[i] = replaceSelf(t.sub(i));
	    }
	    return tf.createTerm(t.op(), subs, 
				 (ArrayOfQuantifiableVariable) null, 
				 null, null);
	}
    }

    public void addSignals(KeYJavaType exc, ProgramVariable v, Term cond){
	if(v != null){
	    getProgramVariableNS().add(v);
	}
	signals = signals.add(new Signals(exc, v, cond));
    }

    public SetOfSignals getSignals(){
	return signals;
    }

    public void addPre(Term t){
	if(pre == null){
	    pre = t;
	}else if(t != null){
	    pre = tf.createJunctorTermAndSimplify(Op.AND, pre, t);
	}
    }

    public void addPost(Term t){
	if(post == null){
	    post = t;
	}else if(t != null){
	    post = tf.createJunctorTermAndSimplify(Op.AND, post, t);
	}
    }

    public void addDiverges(Term t){
	if(diverges == null){
	    diverges = t;
	}else if(t != null){
	    diverges = tf.createJunctorTermAndSimplify(Op.OR, diverges, t);
	}
    }

    public void addAssignable(Term t){
	assignableVariables = assignableVariables.add(t);
    }

    public void addAssignable(SetOfTerm as){
	assignableVariables = assignableVariables.union(as);
    }

    public SetOfTerm getAssignable(){
	return assignableVariables;
    }

    public MethodDeclaration getMethodDeclaration(){
	return pm.getMethodDeclaration();
    }

    public TypeDeclaration getClassDeclaration(){
	return cd;
    }

    /**
     * returns true if this specification demands termination of the method,
     * which means in this case that terminating by throwing an exception
     * is also considered to be a termination.
     */
    public boolean terminates(){
	return diverges == null || diverges.equals(falseTerm);
    }

    /**
     * returns the ProgramVariable that is used for expressing the excetional
     * behavior of a method. 
     */
    public ProgramVariable getExceptionVariable(){
	return excVar;
    }
    
    /**
     * returns true iff the assignable clause contains expressions such as
     * a[i1 .. i2] or o.*.
     */
    public boolean containsQuantifiedAssignableLocation(){
	IteratorOfTerm it = assignableVariables.iterator();
	while(it.hasNext()){
	    if(containsQuantifiedLocationHelp(it.next())){
		return true;
	    }
	}
	return false;
    }

    private boolean containsQuantifiedLocationHelp(Term t){
	if(t.op() instanceof AccessOp){
	    if(t.op() instanceof QuantifiedArrayOp){
		return true;
	    }
	    for(int i=0; i<t.arity(); i++){
		if(containsQuantifiedLocationHelp(t.sub(i))){
		    return true;
		}
	    }
	}
	return false;
    } 

    public SetOfTerm replaceModelFieldsInAssignable(){
	return replaceModelFieldsInAssignable(cSpec);
    }

    public SetOfTerm replaceModelFieldsInAssignable(JMLClassSpec cs){
	if(assignableVariables == SetAsListOfTerm.EMPTY_SET){
	    if("nothing".equals(getAssignableMode())){ 
		return assignableVariables;
	    }else{
		return null;
	    }
	}
	SetOfTerm result = assignableVariables;
	/*	IteratorOfTerm it = assignableVariables.iterator();
	while(it.hasNext()){
	    Term varTerm = it.next();
	    System.out.println("replaceModelFieldsInAssignable3: "+varTerm);
	    Term rep = (Term) cs.getRepresents().get(varTerm);
	    System.out.println("replaceModelFieldsInAssignable4: "+rep);
	    if(rep != null){
		System.out.println("replaceModelFieldsInAssignable5");
		result = result.remove(varTerm).add(rep);
	    }
	    }*/
	return result;
    }

    public void setAssignableMode(String s){
	assignableMode = s;
    }

    public String getAssignableMode(){
	if(services.getImplementation2SpecMap().getModifiers(pm).
	   contains("pure") && 
	   assignableVariables == SetAsListOfTerm.EMPTY_SET){
	    return "nothing";
	}
	if(assignableVariables == SetAsListOfTerm.EMPTY_SET &&
	    assignableMode == null){
	    return "everything";
	}
	// an inconsistent case
	if(assignableVariables != SetAsListOfTerm.EMPTY_SET &&
	   "nothing".equals(assignableMode)){
	    return null;
	}
	return assignableMode;
    }

    /**
     * returns the locally declared precondition
     */
    public Term getPre(){
	return pre;
    }

    /** 
     * returns the precondition + represents for model fields
     * (+ invariants iff withClassSpec = true, + the negation of the diverges
     * clause in order to retrieve a precondition under which the method must 
     * terminate).
     */
    public Term getCompletePre(boolean withClassSpec, boolean allInv,
			       boolean terminationCase){
	Term t = getPre();
	if(withClassSpec){
	    if(allInv && !services.getImplementation2SpecMap().
	       getModifiers(pm).contains("helper")){
		Term ai = UsefulTools.allInvariants(
		    services.getImplementation2SpecMap());
		t = UsefulTools.makeConjunction(t, ai);
	    }else{
		t = cSpec.addClassSpec2Pre(t, pm, cSpec); 
	    }
	}
	if(terminationCase && diverges != null && !falseTerm.equals(diverges)){
	    Term divPre = 
		tf.createJunctorTerm(Op.NOT, new Term[]{diverges}); 
	    divPre = updatePrefix(divPre);
	    t = tf.createJunctorTermAndSimplify(Op.AND, divPre, t);
	}
	t = UsefulTools.addRepresents(t, services, 
				      (ProgramVariable) 
				      cSpec.getInstancePrefix());
	t = updatePrefix(t);
	return t==null? trueTerm : t; 
    }

    /**
     * returns the locally declared postcondition
     */
    public Term getPost(){
	return post;
    }

    
    private Term createdOrNull(ProgramVariable var) {
	final Term varAsTerm = tf.createVariableTerm(var);
	Term result = tf.createEqualityTerm
	    (varAsTerm, 
	     services.getJavaInfo().getNullTerm());
	final ProgramVariable created = services.getJavaInfo().
	    getAttribute(ImplicitFieldAdder.IMPLICIT_CREATED, 
			 services.getJavaInfo().getJavaLangObject());
	result = tf.createJunctorTerm(Op.OR, result, 
	   tf.createEqualityTerm(tf.createAttributeTerm(created, varAsTerm), 
				 trueLitTerm));
	return result;

    }


    /** 
     * returns the postcondition + represents for model fields
     * (+ invariants and constraints iff withClassSpec = true) +
     * created or null for returned objects.
     */
    public Term getCompletePost(boolean withClassSpec, boolean allInv){
	Term result;
	       

	if(oldLVs == null){
	    getPi1();
	}
	if(getPost() == null){
	    result = 
		createModalityTermForEnsures(trueTerm, false, withClassSpec, 
					     allInv);
	}else{
	    result = 
		createModalityTermForEnsures(getPost(), false, withClassSpec, 
					     allInv);
	}
	result = result.sub(0);
	result = addOldQuantifiers(result, lv2old, false, params);
	if(!(pm.getMethodDeclaration() instanceof Constructor)){
	    result = updatePrefix(result);
	}
	if (resultVar != null && 
	    resultVar.sort() instanceof ObjectSort) {
	    result = TermFactory.DEFAULT.createJunctorTerm
		(Op.AND, result, createdOrNull(resultVar));
	}
	return result;
    }

    protected Term updatePrefix(Term t){
	if(!pm.isStatic() && t != null && inheritedPrefix != null){
	    t = tf.createUpdateTerm(
		      tf.createVariableTerm((ProgramVariable) inheritedPrefix),
		      self, t);
	    t = simplifier.simplify(t);
	}
	return t;
    }

    public Term getPi1(){
	if(pi1 == null){
	    lv2old = new TreeMap(comparator);
	    LinkedList l = new LinkedList();
	    pi1 = getPi1Helper(trueTerm, l, term2old);
	    pi1 = getPi1Helper(pi1, l, cSpec.getTerm2Old());
	    pi1 = UsefulTools.addRepresents(pi1, services, 
				      (ProgramVariable) 
					    cSpec.getInstancePrefix());
	    oldLVs = QuantifierPrefixEntry.toUniversalList ( l.iterator () );
	}
	return pi1;
    }

    private Term getPi1Helper(Term pi1, LinkedList l, HashMap map){
	Set sortedKeyset = new TreeSet(comparator);
        sortedKeyset.addAll(map.keySet());
        final Iterator it = sortedKeyset.iterator();
	Term postAndInvTerm = 
	    createModalityTermForEnsures(getPost() == null?
					 trueTerm : getPost(),
					 false, true, false);
	while(it.hasNext()){
	    Term t = (Term) it.next();
	    Term old = (Term) map.get(t);
	    if(UsefulTools.occursIn(old, postAndInvTerm, false)){
		Term t1 = createEqualityTermForOldLV(t, old, l, lv2old);
		if(pi1 == trueTerm){
		    pi1 = t1;
		}else{
		    pi1 = tf.createJunctorTerm(Op.AND,new Term[]{pi1, t1});
		}
	    }
	}
	return pi1;
    }

    public ListOfQuantifierPrefixEntry getOldLVs(){
	if(oldLVs == null){
	    getPi1();
	}
	return oldLVs;
    }

    public HashMap getTerm2Old(){
	return term2old;
    }

    public ProgramVariable getResultVar(){
	if(resultVar == null){
	    resultVar = makeResultVar();
	}
	return resultVar;
    }

    private ProgramVariable makeResultVar(){
	if(pm.getTypeReference()==null){
	    return null;
	}
	ProgramVariable v = 
	    new ProgramVariable(new ProgramElementName("_jmlresult"+
						       (resultVarCount++)),
				pm.getTypeReference().getKeYJavaType());
	progVarNS.add(v);
	return v;
    }

    protected static Term createEqualityTermForOldLV(
	Term t, Term old, LinkedList l, Map lv2old){
	TermFactory tf0 = TermFactory.DEFAULT;
	Term lvTerm = null; 
	Term lvEq = null;
	LogicVariable oldlv=null; 
	if(!(old.op() instanceof ProgramVariable) || 
	   old.sort() == Sort.FORMULA && 
	   !(old.sub(0).op() instanceof ProgramVariable)){
	    Term oldFTerm = (t.sort() == Sort.FORMULA ?
			     old.sub(0) : old);
	    t = tf0.createEqualityTerm(old, t);
	    for(int i = 0; i<oldFTerm.arity(); i++){
		LogicVariable arglv = 
		    new LogicVariable(
			new Name(oldFTerm.sub(i).op().name().
				 toString()+
				 "_lv"),
			oldFTerm.sub(i).sort());
		t = UsefulTools.
		createQuantifierTermWithPV(Op.ALL, oldFTerm.sub(i), arglv, t);
	    }
	    return t;
	}else{
	    if(t.sort() != Sort.FORMULA){
		oldlv = 
		    new LogicVariable(new Name("y"+(logicVarCount++)),
				      t.sort());
		l.add(oldlv);
		lvEq = lvTerm = tf0.createVariableTerm(oldlv);
	    }else{
		oldlv = new LogicVariable(
		    new Name("y"+(logicVarCount++)),
		    falseLitTerm.sort());
		l.add(oldlv);
		lvTerm = tf0.createVariableTerm(oldlv);
		lvEq = tf0.createEqualityTerm(lvTerm,trueLitTerm);
		old = old.sub(0);
	    }
	    Term t1 = tf0.createEqualityTerm(lvEq, t);
	    if(lv2old != null) lv2old.put(lvTerm, old);
	    return t1;
	}
    }


    /** Applies the transformation tau_update to <code>a</code> iff 
     * <code>useQuantifiers == false</code> or tau_old otherwise.
     */
    public static Term addOldQuantifiers(Term a, Map term2old, 
					 boolean useQuantifiers,
					 Namespace params){
	if(a==null) return a;
	final TreeMap param2old = new TreeMap(comparator);
	final Set sortedKeys;
        if (term2old instanceof SortedMap) {
            sortedKeys = term2old.keySet();
        } else {
            sortedKeys = new TreeSet(comparator);
            sortedKeys.addAll(term2old.keySet());
        }
	final Iterator it = sortedKeys.iterator();
	while(it.hasNext()){
	    Term t = (Term) it.next();
	    Term old = (Term) term2old.get(t);
	    if(params != null && params.lookup(t.op().name()) != null){
		param2old.put(t, old);
	    }else{
		a = addOldQuantifierHelp(a, t, old, useQuantifiers);
	    }
	}
	// it is necessary that old-variables associated with parameters are
	// updated first because parameters are always implicitely refered to 
	// in their prestate, which 
	// can lead to nested old-expressions.
	// example: let x be a parameter: 
	// transforming <P>(... \old(x[i]) ...) to 
	// {old1:=old2[i]}{old2:=x}<P>(... old1 ...) is obviously wrong
	// since x has not yet been assigned to old2 when the update
	// {old1:=old2[i]} is applied.
	// the correct transformation would be:
	// {old2:=x}{old1:=old2[i]}<P>(... old1 ...)
	if(params != null && !param2old.isEmpty()){
	    a = addOldQuantifiers(a, param2old, useQuantifiers, null);
	}
	return a;
    }

    private static Term addOldQuantifierHelp(Term a, Term t, Term old, 
					     boolean useQuantifiers){
	TermFactory tf0 = TermFactory.DEFAULT;
	if(UsefulTools.occursIn(old, a, false)){
	    if(useQuantifiers){
		LinkedList l = new LinkedList();
		Term t1 = createEqualityTermForOldLV(t, old, l, null);
		a = TermFactory.DEFAULT.createJunctorTerm(
				           Op.IMP,new Term[]{t1,a});
		if(t.sort() == Sort.FORMULA){
		    old = old.sub(0);
		}
		if(old.op() instanceof ProgramVariable){
		    a = UsefulTools.createQuantifierTermWithPV(
				Op.ALL, old, (LogicVariable) l.getFirst(), a);
		}
	    }else{
		if(t.sort() != Sort.FORMULA && 
		   old.op() instanceof ProgramVariable){
		    a = tf0.createUpdateTerm(old,t,a);
		}else if(t.sort() == Sort.FORMULA && 
			 old.sub(0).op() instanceof ProgramVariable){
		    Term oldVTerm = old.sub(0);
		    LogicVariable oldlv = 
			new LogicVariable(
			      new Name(oldVTerm.op().name().toString()+
				       "_lv"),
			      falseLitTerm.sort());
		    Term lvTerm = 
			tf0.createVariableTerm(oldlv);
		    Term t1 = tf0.createEqualityTerm(lvTerm, trueLitTerm);
		    t = tf0.createEqualityTerm(t, t1); 
		    a = tf0.createJunctorTerm(Op.IMP,new Term[]{t,a});
		    a = UsefulTools.createQuantifierTermWithPV(
					  Op.ALL, oldVTerm, oldlv, a);
		}else{
		    t = createEqualityTermForOldLV(t, old, null, null);
		    a = tf0.createJunctorTerm(Op.IMP,new Term[]{t,a});
		}
	    }
	}
	return a;
    }

    public ReferencePrefix getPrefix(){
	if(pm != null && pm.isStatic()){
	    return cSpec.getStaticPrefix();
	}else{
	    return cSpec.getInstancePrefix();
	}
    }

    public StatementBlock makeMBS(){
	if(mBS == null){
	    ArrayOfParameterDeclaration aopd = pm.getParameters();
	    Expression[] aop = new Expression[pm.getParameters().size()]; 
	    for(int i=0; i<aopd.size(); i++){
		aop[i] = (ProgramVariable) aopd.
		    getParameterDeclaration(i).
		    getVariableSpecification().getProgramVariable();
	    }
	    if(!(pm.getMethodDeclaration() instanceof Constructor)){
		MethodBodyStatement mb = new MethodBodyStatement(
		    pm,
		    pm.isStatic() ? null : getPrefix(),
		    getResultVar(),
		    new ArrayOfExpression(aop));
		mBS = new StatementBlock(mb);
	    }else{
		New n = new New(aop, (TypeReference) cSpec.getStaticPrefix(), null);
		MethodFrame fakeFrame = new MethodFrame
		    (null, 
		     new ExecutionContext((TypeReference) cSpec.getStaticPrefix(), null), 
		     new StatementBlock(new CopyAssignment((ProgramVariable) getPrefix(), n)));
		mBS = new StatementBlock(fakeFrame);
	    }
	}
	return mBS;
    }

    public ProgramMethod getProgramMethod(){
	return pm;
    }
    
    public JavaModelMethod getModelMethod() {
    	return new JavaModelMethod(getProgramMethod(), javaPath);
    }

    protected Term createModalityTermForEnsures(Term post, boolean desugar,
						boolean withInv, 
						boolean allInv){
	ModTermKey key = new ModTermKey(post, desugar, withInv, allInv);
	Term c = (Term) modalityTermForEnsuresCache.get(key);
	if(c != null){
	    return c;
	}
	Term excVarTerm = tf.createVariableTerm(excVar);
	Term excPost = null;
	post = tf.createJunctorTermAndSimplify(
	    Op.IMP, tf.createEqualityTerm(excVarTerm, nullTerm), post);
	IteratorOfSignals it = signals.iterator();
	while(it.hasNext()){
	    Signals sig = it.next();
	    KeYJavaType excType = sig.type();
	    SortDefiningSymbols s = (SortDefiningSymbols)(excType.getSort());
	    InstanceofSymbol func
		= (InstanceofSymbol)s.lookupSymbol(InstanceofSymbol.NAME);
	    Term excInstTerm = tf.createFunctionTerm(func, excVarTerm);
	    excInstTerm = tf.createEqualityTerm(excInstTerm, trueLitTerm);
	    Term post1 = tf.createJunctorTermAndSimplify(
		Op.IMP,	excInstTerm, sig.condition());
	    if(sig.variable() != null){
		post1 = tf.createUpdateTerm(
		    tf.createVariableTerm(sig.variable()), excVarTerm, post1);
	    }
	    if(excPost != null){
		excPost = tf.createJunctorTermAndSimplify(
		    Op.AND, excPost, post1);
	    }else{
		excPost = post1;
	    }
	}
	JavaBlock jb = catchAllJB(desugar);
	if(excPost != null){
	    excPost = tf.createJunctorTermAndSimplify(
		Op.IMP,	tf.createJunctorTerm(
		    Op.NOT, tf.createEqualityTerm(excVarTerm, nullTerm)),
		excPost);
	    post = tf.createJunctorTermAndSimplify(Op.AND, post, excPost);
	}
	if(withInv && !services.getImplementation2SpecMap().
	   getModifiers(pm).contains("helper")){
	    post = cSpec.addClassSpec2Post(post, true, !allInv, pm, cSpec); 
	    if(allInv){
		Term ai = UsefulTools.allInvariants(
		    services.getImplementation2SpecMap());
		if(ai != null){
		    post = tf.createJunctorTermAndSimplify(Op.AND , ai, post);
		}
	    }
	}
	post = updatePrefix(post);
	post = UsefulTools.addRepresents(post, services, 
					 (ProgramVariable) 
					 cSpec.getInstancePrefix());
	if(diverges == null || falseTerm.equals(diverges)){
	    c = tf.createDiamondTerm(jb, post);
	}else{
	    c = tf.createBoxTerm(jb, post);
	}
	modalityTermForEnsuresCache.put(key, c);
	return c;
    }

    protected JavaBlock catchAllJB(boolean desugar){
	JavaBlock catchAllJB;
	if(catchAllStmt == null){
	    ParameterDeclaration param = 
		new ParameterDeclaration(new Modifier[0], new TypeRef(exc),
					 new VariableSpecification(excVar), 
					 false);
	    catchAllStmt = new CatchAllStatement(makeMBS(), param);
	}
	if(desugar){
	    catchAllJB = 
		JavaBlock.createJavaBlock(
		    (StatementBlock) catchAllStmt.desugar());
	}else{
	    catchAllJB = JavaBlock.createJavaBlock(new StatementBlock(
						       catchAllStmt));
	}
	return catchAllJB;
    }

    /**
     * @param t1 the precondition.
     * @param jb the JavaBlock used in the modality.
     */
    protected Term createDiverges(){
	if(diverges != null && !falseTerm.equals(diverges)){
	    Term div = tf.createDiamondTerm(catchAllJB(true), trueTerm);
	    Term divPre = 
		tf.createJunctorTerm(Op.NOT, new Term[]{diverges}); 
	    divPre = updatePrefix(divPre);
	    return tf.createJunctorTermAndSimplify(Op.IMP, divPre, div);
	}
	return trueTerm;
    }

    protected Term updateParameters(ArrayOfParameterDeclaration oldParams, 
				    ArrayOfParameterDeclaration newParams, 
				    Term target){
	if(target == null) return null;
	for(int i=0; i<newParams.size(); i++){
	    ProgramVariable oldP = (ProgramVariable) oldParams.
		getParameterDeclaration(i).
		getVariableSpecification().getProgramVariable();
	    ProgramVariable newP = (ProgramVariable) newParams.
		getParameterDeclaration(i).
		getVariableSpecification().getProgramVariable();
	    target = tf.createUpdateTerm(tf.createVariableTerm(oldP), 
					 tf.createVariableTerm(newP),
					 target);
	}
	return simplifier.simplify(target);
    }
	
    /** Creates a DL formula from this method specification:
     * ((pre & inv) -> [{methodbody}]post) & ((pre & inv & !diverges) -> <{methodbody}>post)
     * (This is the DL formula that is needed for an "ensured postcondition 
     * proofobligation")
     * @param withInvariant: iff true the invariant of the containing type
     * is added to the postcondition which is needed for JMLPostAndInvPOs 
     * created by JMLMethodContracts.
     * @param allInvariants: iff true all existing invariants for every 
     * existing type are added to the precondition (and postcondition if 
     * withInvariant is also true).
     */
    public Term createDLFormula(boolean withPostInvariant, 
				boolean allInvariants){
	Term result,t1,t2;
	result = t2 = null;
	t1 = getPre();
	if(allInvariants && !services.getImplementation2SpecMap().
	   getModifiers(pm).contains("helper")){
	    Term ai = UsefulTools.allInvariants(
		services.getImplementation2SpecMap());
	    t1 = UsefulTools.makeConjunction(t1, ai);
	}else{
	    t1 = cSpec.addClassSpec2Pre(t1, pm, cSpec);
	}
	t2 = getPost();
	if(t2 == null){
	    result= createModalityTermForEnsures(trueTerm, true, 
						 withPostInvariant,
						 allInvariants);
	}else{
	    result= createModalityTermForEnsures(t2, true,
						 withPostInvariant,
						 allInvariants);
	}
	if(diverges != null && !falseTerm.equals(diverges)){
	    result = tf.createJunctorTermAndSimplify(
		Op.AND,	result, createDiverges()); 
	}
	if(t1 != null){
	    result = tf.createJunctorTermAndSimplify(Op.IMP, t1, result);
	}
	result = addOldQuantifiers(result, term2old, false, params);
	if(cSpec!=null){
	    result = addOldQuantifiers(result, cSpec.getTerm2Old(), false, 
				       null);
	}
	if(!pm.isStatic()){
	    result = updatePrefix(result);
	}
	result = UsefulTools.addRepresents(result, services, 
					   (ProgramVariable) 
					   cSpec.getInstancePrefix());
/*	if(UsefulTools.purityCheck(result, 
				   services.getImplementation2SpecMap())
	    != null){
	    throw new RuntimeException("Nonpure method "+ 
				       UsefulTools.purityCheck(result, 
					 services.getImplementation2SpecMap())+
				       " used in the "+
				       "specification for method "+
				       pm.getName());
				       }*/
	if(!pm.isStatic()){
	    result = UsefulTools.quantifySelf(result, pm.getMethodDeclaration(), 
	    		                          (ProgramVariable)getPrefix());
	}
	return UsefulTools.quantifyProgramVariables(result, 
			    params.allElements().iterator(), Op.ALL, services);
    }

    protected class ModTermKey{ 

	private int hashValue;
	
	private Term post;
	private boolean desugar, withInv, allInv;

	public ModTermKey(Term post, boolean desugar, boolean withInv, 
			  boolean allInv){
	    hashValue = 13*post.hashCode() + 3*(desugar ? 1 : 0) +
		5*(withInv ? 1 : 0) + 7*(allInv ? 1 : 0);
	    this.post = post;
	    this.desugar = desugar;
	    this.withInv = withInv;
	    this.allInv = allInv;
	}

	public int hashCode(){
	    return hashValue;
	}

	public boolean equals(Object o){
	    if(o == null || !(o instanceof ModTermKey)) return false;
	    ModTermKey k = (ModTermKey) o;
	    return k.desugar==desugar && k.withInv==withInv && k.allInv==allInv
		&& post.equals(k.post);
	}	
    }
    
    public String getJavaPath() {
    	return javaPath;
    }

}
