// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.logic.op;

import java.lang.ref.WeakReference;
import java.util.WeakHashMap;

import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.sort.ArraySort;
import de.uka.ilkd.key.logic.sort.GenericSort;
import de.uka.ilkd.key.logic.sort.ProgramSVSort;
import de.uka.ilkd.key.logic.sort.Sort;

public class QuantifiedArrayOp extends ArrayOp {

    /** 
     * The pool of already created array operators. Ensures
     * 'singleton' property of array operators.
     */
    private static final WeakHashMap operatorPool = new WeakHashMap(50);
    
    /**
     * returns and if neccessary creates the shadowed array access operator 
     * for the given array sort.
     * 
     * @param arraySort the Sort to which this operator is applicable
     *  must be of kind {@link ArraySort} or {@link ProgramSVSort}
     * @return the array access operator for the given array sort
     */
    public static QuantifiedArrayOp getQuantifiedArrayOp(Sort arraySort) {
        Debug.assertTrue(arraySort instanceof ArraySort ||
                arraySort instanceof ProgramSVSort || 
                arraySort instanceof GenericSort,
                arraySort + " is no allowed array sort.");
        WeakReference arrayAccessOpReference = (WeakReference) operatorPool
                .get(arraySort);
        if (arrayAccessOpReference == null
                || arrayAccessOpReference.get() == null) {
            // wrapping attributeOp in a weak reference is necessary as
            // it contains a strong reference to its key
            arrayAccessOpReference = 
                   new WeakReference(new QuantifiedArrayOp(arraySort));
            operatorPool.put(arraySort, arrayAccessOpReference);
        }
        return (QuantifiedArrayOp) arrayAccessOpReference.get();
    } 
    
    private QuantifiedArrayOp(Sort arraySort) {
        this(new Name("quantifiedArrayAccess(" + arraySort +")"), arraySort);
    }
    
    protected QuantifiedArrayOp(Name name, Sort arraySort) {
        super(name, arraySort);
    }
    
   
    public int arity(){
	return 3;
    }

    public de.uka.ilkd.key.java.Expression convertToProgram(Term t, 
							    ExtList el) {
	return null;
    }

}
    
