package de.uka.ilkd.key.logic.op;

import java.lang.ref.WeakReference;
import java.util.WeakHashMap;

import de.uka.ilkd.key.logic.sort.GenericSort;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.sort.ArraySort;
import de.uka.ilkd.key.logic.sort.ProgramSVSort;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.util.Debug;

public class ShadowedQuantifiedArrayOp 
    extends QuantifiedArrayOp implements ShadowedOperator{
    /** 
     * The pool of already created array operators. Ensures
     * 'singleton' property of array operators.
     */
    private static final WeakHashMap operatorPool = new WeakHashMap(50);
    
    /**
     * returns and if neccessary creates the shadowed array access operator 
     * for the given array sort.
     * 
     * @param arraySort the Sort to which this operator is applicable
     *  must be of kind {@link ArraySort} or {@link ProgramSVSort}
     * @return the array access operator for the given array sort
     */
    public static ShadowedQuantifiedArrayOp getShadowedQuantifiedArrayOp(Sort arraySort) {
        Debug.assertTrue(arraySort instanceof ArraySort ||
                arraySort instanceof ProgramSVSort || 
                arraySort instanceof GenericSort,
                arraySort + " is no allowed array sort.");
        WeakReference arrayAccessOpReference = (WeakReference) operatorPool
                .get(arraySort);
        if (arrayAccessOpReference == null
                || arrayAccessOpReference.get() == null) {
            // wrapping array access operator in a weak reference is necessary as
            // it contains a strongimport de.uka.ilkd.key.rule.metaconstruct.MetaShadow; reference to its key
            arrayAccessOpReference = 
                   new WeakReference(new ShadowedQuantifiedArrayOp(arraySort));
            operatorPool.put(arraySort, arrayAccessOpReference);
        }
        return (ShadowedQuantifiedArrayOp) arrayAccessOpReference.get();
    } 
    
    private ShadowedQuantifiedArrayOp(Sort arraySort) {
        super(new Name("Shadowed_Quantified_Arrayoperator(" + arraySort + ")"), arraySort);     
    }

    public int arity(){
        return 4;
    }
        
    public Term getTransactionNumber(Term t) {     
        return t.sub(3);
    }

    /**
     * @return true if the operator is a shadowed 
     */
    public boolean isShadowed() {
        return true;
    }
}
