// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

package de.uka.ilkd.key.logic.sort;


import java.util.WeakHashMap;

import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.op.*;



public class ArraySortImpl extends AbstractCollectionSort implements ArraySort{

    private static WeakHashMap aSH = new WeakHashMap();
    
    private final ArrayOfSort commonJavaSorts;  
    
    private SetOfSort extendsSorts;


    private ArraySortImpl(Sort elemSort, SetOfSort extendsSorts, 
			  Sort objectSort, 
                          Sort cloneableSort, 
                          Sort serializableSort) {
	super(elemSort.name()+"[]",elemSort);
	
        Sort[] commons = new Sort[3];
        commons[0] = objectSort;
	commons[1] = cloneableSort;
        commons[2] = serializableSort;
        commonJavaSorts = new ArrayOfSort(commons);
	// uncommented because of ASMKeY:
	//	Debug.assertTrue(extendsSorts!=SetAsListOfSort.EMPTY_SET);
	this.extendsSorts = extendsSorts;
    }

    /**
     * @return an object of this class with elementSort().equals(p),
     * or null if such an object cannot be constructed (as p is an
     * incompatible sort).
     */
    public Sort cloneFor ( Sort p ) {
	return getArraySort ( p, commonJavaSorts.getSort(0), 
                commonJavaSorts.getSort(1), commonJavaSorts.getSort(2));
    }

    /**
     * @return the sorts of the predecessors of this sort
     */
    public SetOfSort extendsSorts() {
	return extendsSorts;
    }

    /** 
     * returns elemSort([])^n.
     * @param sort serializable the Sort representing interface java.lang.Serializable
     */
    public static Sort getArraySortForDim(Sort elemSort, int n, 
					  Sort objectSort, 
					  Sort cloneableSort, 
                                          Sort serializableSort){
	Sort result = elemSort;
	while(n>0){
	    result = getArraySort(result, objectSort, cloneableSort, serializableSort);
	    n--;
	}
	return result;
    }
    
    /**
     * returns the ArraySort to the given elementsort. This method ensures that
     * only one ArraySort-object exists for each Arraysort.
     */
    public static ArraySort getArraySort(Sort elemSort, 
                                         Sort objectSort,
                                         Sort cloneableSort,
                                         Sort serializableSort){
	if (aSH.containsKey(elemSort)) {
	    return (ArraySort) aSH.get(elemSort);
	} else { 
        // HACK: this simple handling of sort creation does not treat
        // depending symbols (i.e. ArrayOfS::instance)
	    SetOfSort localExtendsSorts = getArraySuperSorts(elemSort, objectSort,
							     cloneableSort, 
                                                             serializableSort);
	    ArraySort aS = new ArraySortImpl(elemSort, localExtendsSorts,
					     objectSort, cloneableSort, 
                                             serializableSort);
	    aSH.put(elemSort,aS);
	    return aS;
	}
    }

    private static SetOfSort getArraySuperSorts(Sort elem, 
						Sort objectSort,
						Sort cloneableSort,
                                                Sort serializableSort){
	int i = 1;
	while(elem instanceof ArraySort){
	    elem = ((ArraySort) elem).elementSort();
	    i++;
	}
	SetOfSort superSorts = elem.extendsSorts();
	if(elem instanceof PrimitiveSort || elem == objectSort){
	    i--;
	    superSorts = superSorts.add(cloneableSort).add(serializableSort);
	}
	if(i>0){
	    final IteratorOfSort it = superSorts.iterator();
	    superSorts = SetAsListOfSort.EMPTY_SET;
	    while(it.hasNext()){
		Sort s = it.next();
		superSorts = 
		    superSorts.add(getArraySortForDim(s, i, 
						      objectSort, 
						      cloneableSort, 
                                                      serializableSort));
	    }
	}
	return superSorts;
    }

    /**
     * Lookup the symbol of kind "p_name", which is a sort
     * independent identifier for this symbol
     * @return Symbol with (kind) name "p_name"
     * ("ret.getKind().equals(p_name)"), null if no such object exists
     */
    public SortDependingSymbol lookupSymbol  ( Name      p_name ) {
        return definedSymbols.get ( p_name );
    }
}
