// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

//
// Offers the functionality of tudresden.ocl.ASTViewer 
// but decoupled from the graphical user interface issues
//

package de.uka.ilkd.key.ocl.tudresden;


// Imports from the original parser
import tudresden.ocl.parser.OclParserException;
import tudresden.ocl.parser.node.Start;
import tudresden.ocl.parser.parser.ParserException;
import tudresden.ocl.check.TypeQueryable;
import tudresden.ocl.*;

import de.uka.ilkd.key.util.KeYResourceManager;
import de.uka.ilkd.key.ocl.ocldltranslator.*;
import de.uka.ilkd.key.ocl.ocldltranslator.detailedocl.*;
import de.uka.ilkd.key.ocl.ocldltranslator.test.*;
import de.uka.ilkd.key.casetool.ModelManager;
import de.uka.ilkd.key.casetool.HashMapOfClassifier;

import java.io.*;
import java.net.*;
import javax.swing.tree.*;

public class TUDresdenParserAdapter{

    OclTree oclTree;
    // attributes to store syntax errors found by the OCL parser
    private String errorMessageToUser = "";
    private boolean errorOccurred = false;


    private void printURLStream(URL url) {
	try {
	    InputStream ins=url.openStream();
	    char c;
	    while ((c=(char)ins.read())!=-1) {
		System.out.print(c);
	    }
	    System.out.println();
	    ins.close();
	} catch (Exception e) {
	    System.out.println("Fehler: "+e);
	}
    }

    public OclTree doParse1(String constraintIdent, ModelManager man, 
			    String location,
			    String constraint, 
			    String containingPackage){
	return doParse1(constraintIdent, man, location, null,
			constraint, containingPackage);
    }

    public OclTree doParse1(String constraintIdent, 
			    ModelManager man, String location, 
			    String modelId, String constraint, 
			    String containingPackage){
    	oclTree = null;

	// parsing the constraint
	try {
	    // Modify the constraint in order to get a parsable
	    // version of the constraint.  This is mainly necessary
	    // for constraints on constructors (that start with a
	    // capital letter!)
	    ConstraintModifier cModifier = new ConstraintModifier();
	    String modifiedConstraint 
		= cModifier.modifyConstraintText(constraint);
	    tudresden.ocl.check.types.ModelFacade mf 
		= man.getModelFacade(location, modelId);
	   
	    oclTree=OclTree.createTree(modifiedConstraint, mf, 
				       new KeyTypeCheckerFactory());
	    if (oclTree.getRoot()!=null) {
		displayTree(constraintIdent, oclTree.getRoot(), oclTree);
		//createTextFile();
	    }
    	} catch (OclParserException e) {
	    showMessage("OclParserException occurred");
	    showMessage(e);
	    setErrorMessageToUser(constraintIdent,
				  "Syntax Error. OCLParser Exception occurred",
				  true);
	    e.printStackTrace();
	    //doLex();
	    return oclTree;
	} catch (java.io.IOException e) {
	    throw new IllegalArgumentException("Error reading model file "
					       +"\n"+e.getMessage());
	}
	return oclTree;
    }

    // displayTree generates the errorMessages iff a syntactical 
    // error occurred
  protected void displayTree(String constraintIdent, Start root, TypeQueryable tq) {
      Visualizer vis=new Visualizer(true, tq);
      if (root == null){showMessage("doParse1(): displayTree(): root is null");}
      root.apply(vis);
      if (vis == null){showMessage("doParse1(): displayTree(): vis is null");}
      //if (tree==null) 
      //tree=new JTree( vis.getRootNode() );
      //} else {
      TreeModel model=new DefaultTreeModel(vis.getRootNode());
      //  tree.setModel( model );
      //makeVisible(tree, model, vis.getRootNode());
      if (vis.getErrorMessage() == null){
	  setErrorMessageToUser(constraintIdent,"No syntax error found", false);
      }else{
	  setErrorMessageToUser(constraintIdent," Syntax error: " + 
				vis.getErrorMessage(), true );
      }
      //}
  }

    //   public void doLex() {
    //       //    Lexer lexer=new Lexer(
    //     tudresden.ocl.parser.lexer.Lexer lexer=new tudresden.ocl.parser.lexer.Lexer(
    //       new PushbackReader( new StringReader(text.getText()) )
    //     );
    //     Token t;
    //     DefaultMutableTreeNode dtm=new DefaultMutableTreeNode("lexer output");
    //     try {
    //       do {
    //         t=lexer.next();
    //         String text=t.getClass().getName()+" "+t.toString();
    //         dtm.add(new DefaultMutableTreeNode(text));
    //       } while (! (t instanceof EOF) );
    //     }
    //     catch (Exception e) {}
    //     tree.setModel(
    //       new DefaultTreeModel(dtm)
    //     );
    //   }

    public void createTextFile() {
	//     if (oclTree!=null) {
	//       try {
	//         ASTTextGenerator textGen=new ASTTextGenerator();
	//         oclTree.apply(textGen);

	// 	FileWriter fw=new FileWriter("d:\\temp\\ocl.txt");
	//         fw.write(textGen.getText());
	//         fw.close();
	//       }
	//       catch (IOException e) {
	// 	  //        message.setText("error writing to file: "+e.getMessage());
	//         messageToUser("error writing to file: "+e.getMessage());
	//        }
	//     }
    }

    protected void showMessage(Exception e) {
	String s=e.getMessage();
	if (s==null) {
	    //      message.setText(e.getClass().getName()+" without message");
	    messageToUser(e.getClass().getName()+" without message");
	    e.printStackTrace(System.out);
	    return;
	} else {
	    showMessage(s);
	}
	if (e instanceof ParserException) {
	    ParserException pe=(ParserException)e;
	    String index=s.substring( s.indexOf(",")+1, s.indexOf("]") );
	    int ix=Integer.parseInt(index);
	    //text.setCaretPosition(ix);
	    //text.setSelectionStart(ix);
	    //text.setSelectionEnd(ix+1);
	    messageToUser(" Set Selection on Position " + ix);
	}
    }

    protected void showMessage(String s) {
	if (s==null || s.equals("")) {
	    //      message.setText(" ");
	} else {
	    StringBuffer sb=new StringBuffer();
	    for (int i=0; i<s.length(); i++) {
		char c=s.charAt(i);
		if (! Character.isISOControl(c) ) {
		    sb.append(c);
		}
	    }
	    //      message.setText(sb.toString());
	    messageToUser(sb.toString());
	}
    }


    private void messageToUser(String msg){
	System.err.println("Msg from TUDresdenParserAdapter: " + msg);
    }

    private void setErrorMessageToUser(String culprit, String msg, 
				       boolean occurrenceFlag){
	errorMessageToUser = culprit + ": "+ msg;
	errorOccurred = occurrenceFlag;
    }


    public String getErrorMessageToUser(){
    	return errorMessageToUser;
    }

    public boolean getErrorOccurred(){
    	return errorOccurred;
    }

}
