//This file is part of KeY - Integrated Deductive Software Design
//Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                      Universitaet Koblenz-Landau, Germany
//                      Chalmers University of Technology, Sweden
//
//The KeY system is protected by the GNU General Public License. 
//See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.proof.init;

import java.io.File;

import javax.swing.JOptionPane;

import de.uka.ilkd.key.gui.JMLSpecBrowser;
import de.uka.ilkd.key.proof.ProofAggregate;
import de.uka.ilkd.key.util.ProgressMonitor;

/**
 * This class extends the JavaInput by opening the JML Specification Browser
 * after loading the java source files and contained JML specifications.
 */
public class JavaInputWithJMLSpecBrowser extends JavaInput {

    /**
     * @param name    name of the input, will be the name of the created environment
     * @param path    path of the java files
     * @param monitor ProgressMonitor for showing progress while loading
     */
    public JavaInputWithJMLSpecBrowser(String name, File path, ProgressMonitor monitor) {
        super(name, path, monitor);
    }
    
    /* (non-Javadoc)
     * @see de.uka.ilkd.key.proof.init.ProofOblInput#getPO()
     */
    public ProofAggregate getPO(){
	String javaPath = null;
	try{
	    javaPath = getJavaPath();
	}catch(ProofInputException e){
	    initConfig.getServices().getExceptionHandler().reportException(e);
	}
	JMLProofOblInput jmlPO = null;
	int option = JOptionPane.YES_OPTION;
	do{
	    JMLSpecBrowser sb = new JMLSpecBrowser(initConfig.getServices(),
						   javaPath);
	    jmlPO = sb.getPOI();
	    if(jmlPO == null){
		option = JOptionPane.showConfirmDialog
		    (null, "Do you really want to abort loading the proof?", 
		     "Abort Loading", 
		     JOptionPane.YES_NO_OPTION);
	    }
	}while(option == JOptionPane.NO_OPTION && jmlPO == null);
	if(jmlPO == null){
	    throw new RuntimeException("No Proof");
	}else{
	    jmlPO.setConfig(getInitConfig());
	}
	try{
	    jmlPO.readProblem(ModStrategy.NO_GENSORTS);
	}catch(ProofInputException e){
	    initConfig.getServices().getExceptionHandler().reportException(e);
	}
	ProofAggregate pl = jmlPO.getPO();
	getInitConfig().getProofEnv().registerProof(jmlPO, pl);
	return pl;
    }

}
