// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.proof.mgt;

import java.util.List;

import de.uka.ilkd.key.casetool.ReprModelMethod;
import de.uka.ilkd.key.jml.JMLLemmaMethodSpec;
import de.uka.ilkd.key.jml.JMLMethodSpec;
import de.uka.ilkd.key.logic.op.Modality;
import de.uka.ilkd.key.logic.op.Op;
import de.uka.ilkd.key.proof.ProofAggregate;
import de.uka.ilkd.key.proof.init.JMLPostAndInvPO;
import de.uka.ilkd.key.proof.init.ProofOblInput;
import de.uka.ilkd.key.rule.Taclet;

public class JMLMethodContract extends AbstractContract 
                               implements MethodContract{

    private ReprModelMethod meth;    
    private JMLMethodSpec spec;
    private Modality modality;
    private String typeDescr;
    private Taclet lemma;

    //iff true all existing invariants are added to the pre- and postcondition
    //of the method contract, iff false only the invariants of the containing
    //type and the instance invariants (if meth is not static) for the 
    //corresponding object are added. History constraints of the current type
    //are always added.
    public static final boolean allInvariants = true;

    public JMLMethodContract(ReprModelMethod meth, JMLMethodSpec spec, 
			     Modality modality, String typeDescr) {
	this.meth=meth;
	this.spec = spec;
	this.modality = modality;
	this.typeDescr=typeDescr;
    }


    public void setProofEnv(ProofEnvironment env) {
	if (this.env==null) {
	    this.env=env;
	} else if (this.env!=env) {
	    throw new IllegalStateException("Method Contract may belong to "
					    +"only one environment.");
	}
    }

    public ProofEnvironment getProofEnv() {
	return env;
    }

    public ReprModelMethod getModelMethod() {
	return meth;
    }

    public Taclet getLemmaAndRegister() {
	if ((lemma==null)&&
	    !((JavaModelMethod) meth).isConstructor()) {
	    JMLMethodLemmaFactory ms 
		= new JMLMethodLemmaFactory(
		    env.getInitConfig(), spec, typeDescr, 
                    ModalityClass.getSchemaModality(modality)); 
	    lemma = ms.getTaclet();
	    if (lemma!=null) {
		env.registerRule(lemma, new RuleJustificationBySpec(this));
	    }
	    //	    spec.clearModalityTermForEnsuresCache();
	}
	return lemma;
    }

    public List getProofs() {
	return proofs;
    }

    public void addCompoundProof(ProofAggregate pl) {
	proofs.add(pl);
    }

    public String getPreText() {
	return "";
    }

    public String getPostText() {
	return "";
    }

    public Modality getModality() {
	return ModalityClass.getNormReprModality(modality);
    }

    public Object getObjectOfContract() {
	return spec.getModelMethod();
    }

    public ProofOblInput getProofOblInput() {
	return new JMLPostAndInvPO(spec, env.getJavaModel().getModelDir(),
				   allInvariants);
    }

    public void removeCompoundProof(ProofAggregate pl) {
	proofs.remove(pl);
    }

    public String toString() {
	return "[JML] context " + meth + (spec == null? "" :
					  "\npre: "+spec.getCompletePre(
					      true, allInvariants, 
					      modality == Op.DIA)+
					  "\npost: "+spec.getCompletePost(
					      true, allInvariants)+
					  "\nmodifies: " + spec.
					  replaceModelFieldsInAssignable() +
					  "\npi1: "+ ((JMLLemmaMethodSpec) 
					  spec).getPi1() +
					  "\nmodality: "+modality);
    }

    public JMLMethodSpec getSpec(){
	return spec;
    }

    public boolean equals(Object cmp) {
	if (!(cmp instanceof JMLMethodContract)) return false;
	JMLMethodContract mc = (JMLMethodContract)cmp;
	return meth.equals(getObjectOfContract()) && 
	    getModality() == mc.getModality() &&
	    getSpec().equals(mc.getSpec()); 
    }

    public int hashCode() {
	int result = 0;
	result = 37 * result + meth.hashCode(); 
	result = 37 * result + getModality().hashCode();
	result = 37 * result + getSpec().hashCode();
	return result;
    }

}
