// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.proof.mgt;

import de.uka.ilkd.key.java.ProgramElement;
import de.uka.ilkd.key.java.Statement;
import de.uka.ilkd.key.jml.JMLLemmaMethodSpec;
import de.uka.ilkd.key.jml.JMLMethodSpec;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.SetOfTerm;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.op.Operator;
import de.uka.ilkd.key.logic.op.Op;
import de.uka.ilkd.key.proof.init.InitConfig;
import de.uka.ilkd.key.rule.RuleSet;
import de.uka.ilkd.key.rule.Taclet;

public class JMLMethodLemmaFactory extends AbstractMethodLemmaFactory {

    private InitConfig initConfig;
    // a set of JMLMethodSpecs
    private JMLMethodSpec spec = null;
    private JMLLemmaMethodSpec lemmaspec = null;
    private String suffix = null;
    private Operator modality;
    private AbstractMethodLemmaFactory.ModifiesElement[] modElems;


    public JMLMethodLemmaFactory(InitConfig initConfig, JMLMethodSpec spec, 
				 String suffix, Operator modality) {
	super(initConfig.getServices());
	this.initConfig = initConfig;	
	this.spec = spec;
	this.suffix = suffix;
	this.modality = modality;	
	this.lemmaspec = spec;
	
    }

    protected Operator getModality() {
	return modality;
    }

    public Taclet getTaclet() {
	if(lemmaspec != null){
	    return super.getTaclet();
	}else{
	    return null;
	}
    }

    protected RuleSet getRuleSet() {
	return (RuleSet)getInitConfig().ruleSetNS().lookup
	    (new Name("use_method_specs"));
    }

    public InitConfig getInitConfig(){
	return initConfig;
    }

    /** precondition which is called "PRE" by Sebastian
     * Katz's Studienarbeit */
    protected Term getPreCond(){
	return lemmaspec.getCompletePre(true, JMLMethodContract.allInvariants,
					modality == ModalityClass.
					getSchemaModality(Op.DIA));
    } 
    
    /** extra quantifiable variables created by the transformation necessary
     * after the Studienarbeit of Sebastian Katz */
    protected ListOfQuantifierPrefixEntry getExtraVars(){
	return lemmaspec.getOldLVs();
    } 

    protected ListOfQuantifierPrefixEntry getQuantifierPrefix () {
        return SLListOfQuantifierPrefixEntry.EMPTY_LIST;
    }

    /** extra precondtion created by the transformation necessary
     * after the Studienarbeit of Sebastian Katz 
     * there this is called "pi1" */
    protected Term getExtraPre(){
	return lemmaspec.getPi1();
    }
    
    /** post condition of the method spec
     * this is called "pi2" in Sebastian Katz's Studienarbeit */
    protected Term getPostCond(){
	return lemmaspec.getCompletePost(true,JMLMethodContract.allInvariants);
    } 

    /** modifies elements as a set of terms */
    protected AbstractMethodLemmaFactory.ModifiesElement[] getModifies(){        
        if (modElems==null) {
            SetOfTerm modifies = lemmaspec.replaceModelFieldsInAssignable();
            modElems = AbstractMethodLemmaFactory.TermModifiesElement.create(modifies);
        }
        return modElems;                
    }

    /** statement before the transformation so we can get the real variables (no schema variables) */
    protected ProgramElement getProofStatement(){
	return lemmaspec.getProofStatement();
    }
    
    /** string which is printed out a couple of times and given in the
     * constructor of the classes that extend this abstract class */
    protected String getSuffix(){
	return suffix;
    }

    protected Name getTacletName(Statement transformedStatement) {
	String result = super.getTacletName(transformedStatement).toString();
	String specString = spec.toString();
	result += 
	    "_"+specString.substring(0, specString.indexOf("for method")-1) +
	    "_in_class_"+spec.getClassDeclaration().getName(); 
	result += getPostCond().toString().substring(0,3)+"_"+
	    getExtraPre().toString().substring(0,3)+"_"+
	    getPreCond().toString().substring(0,3)+ spec.id();
	return new Name(result);
    }

}
