// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//

/** this class is used to parse in Taclet from a file that are used by tests */
package de.uka.ilkd.key.rule;

import java.io.File;
import java.io.StringReader;

import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.ProgramElement;
import de.uka.ilkd.key.java.Recoder2KeY;
import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.java.StatementBlock;
import de.uka.ilkd.key.java.declaration.LocalVariableDeclaration;
import de.uka.ilkd.key.logic.ConstrainedFormula;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Namespace;
import de.uka.ilkd.key.logic.NamespaceSet;
import de.uka.ilkd.key.logic.Semisequent;
import de.uka.ilkd.key.logic.Sequent;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.TermFactory;
import de.uka.ilkd.key.logic.op.Function;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.parser.KeYLexer;
import de.uka.ilkd.key.parser.KeYParser;
import de.uka.ilkd.key.parser.ParserMode;
import de.uka.ilkd.key.pp.AbbrevMap;
import de.uka.ilkd.key.proof.Node;
import de.uka.ilkd.key.proof.Proof;
import de.uka.ilkd.key.proof.ProofAggregate;
import de.uka.ilkd.key.proof.TacletIndex;
import de.uka.ilkd.key.proof.init.KeYFileForTests;
import de.uka.ilkd.key.proof.init.ProblemInitializer;

public class TacletForTests {

    private TacletForTests() {}

    public static final String testRules = System.getProperty("key.home")+
        File.separator+"system"+File.separator+"proofExamples"+
        File.separator+"testrules.key";
    public static String standardFile = testRules;
    public static NamespaceSet nss = new NamespaceSet();
    public static AbbrevMap scm = new AbbrevMap();
    public static TacletIndex rules= null;
    public static ProofAggregate problems;
    public static ProblemInitializer pi=null;
    public static Services services;
    public static Recoder2KeY r2k;
    public static File lastFile=null;

    
    public static void parse(File file) {
	try {	    
	    if (!file.equals(lastFile)) {
		ProblemInitializer.BASE_CONFIG = null;
		pi = ProblemInitializer.
		    problemInitializerForTests(new KeYFileForTests("Test", file), 
		                               true); 
		problems = pi.setUpTest();
		nss      = pi.initConfig().namespaces(); 
		rules    = pi.initConfig().createTacletIndex();
		services = pi.initConfig().getServices();
		lastFile = file;
	    }
	} catch (Exception e) {
	    System.err.println("Exception occured while parsing "+file+"\n");
	    e.printStackTrace();
	    System.exit(-1);
	}

    }

    public static Services services() {
	if (services == null) parse();
	return services;
    }

    public static JavaInfo javaInfo() {
	return services ().getJavaInfo ();
    }

    public static JavaInfo getJavaInfo() {
	return javaInfo();
    }

    public static void setStandardFile(String filename) {
	standardFile=filename;
    }

    public static ProofAggregate problems() {
	return problems;
    }

    public static void parse(String filename) {
	parse(new File(filename));
    }

    public static void parse() {
	parse(standardFile);
    }
    
    public static NoPosTacletApp getTaclet(String filename, String name) {
	parse(filename);
	return rules.lookup(new Name(name));
    }

    public static NoPosTacletApp getTaclet(String name) {
	return rules.lookup(new Name(name));
    }
    
    public static AbbrevMap getAbbrevs(){
        return scm;
    }

    public static Namespace getSorts() {
	return nss.sorts();
    }

    public static TacletIndex getRules() {
	return rules;
    }


    public static Namespace getHeuristics() {
	return nss.ruleSets();
    }

    public static Namespace getFunctions() {
	return nss.functions();
    }


    public static Namespace getVariables() {
	return nss.variables();
    }

    public static Namespace getProgramVariables() {
	return nss.programVariables();
    }

    public static NamespaceSet getNamespaces() {
	return nss;
    }

    public static Function funcLookup(String name) {
	return (Function)getFunctions().lookup(new Name(name));
    }

    public static SchemaVariable svLookup(String name) {
	return (SchemaVariable)getVariables().lookup(new Name(name));
    }

    public static Sort sortLookup(String name) {
	return (Sort)getSorts().lookup(new Name(name));
    }

    public static Term parseTerm(String termstr, Services services) {
	if (termstr.equals("")) return null;
	try {	    
	    StringReader br   = new StringReader(termstr);
	    KeYParser parser = new KeYParser(ParserMode.TERM,
	                                     new KeYLexer(br,null), 
					     "No file. " +
					     "TacletForTests.parseTerm("
					     +termstr+")",
					     TermFactory.DEFAULT, 
					     new Recoder2KeY(services, nss),
					     services, nss,
					     TacletForTests.getAbbrevs());
	    return parser.term();
	} catch (Exception e) {
	    System.err.println("Exception during parsing!");
	    e.printStackTrace();
	    return null;
	}
	
    }

    public static Term parseTerm(String termstr, NamespaceSet set) {
	if (termstr.equals("")) return null;
	try {	    
	    StringReader br=new StringReader(termstr);
	    KeYParser parser = new KeYParser(ParserMode.TERM,
	                                     new KeYLexer(br,null), 
					     "No file. " +
					     "TacletForTests.parseTerm("
					     +termstr+")",
					     TermFactory.DEFAULT, 
					     new Recoder2KeY(services(), set),
					     services(),
					     set,
					     new AbbrevMap());
	    return parser.term();
	} catch (Exception e) {
	    System.err.println("Exception during parsing!");
	    e.printStackTrace();
	    return null;
	}
	
    }

    public static Term parseTerm(String termstr) {
	return parseTerm(termstr, services());
    }

    public static ProgramElement parsePrg(String prgString) {
	r2k = new Recoder2KeY(services(), new NamespaceSet());
	return r2k.readBlockWithEmptyContext(prgString).program();
    }

    public static ProgramElement parsePrg(String prgString, 
					  Services ownServices) {
	r2k = new Recoder2KeY(ownServices, new NamespaceSet());
	return r2k.readBlockWithEmptyContext(prgString).program();
    }

    public static ProgramElement parseExpr(String type, String prgString) {
	r2k=new Recoder2KeY(new Services(), new NamespaceSet());

	ProgramElement pe=r2k.readBlockWithEmptyContext("{"+type
	       					+" x="+prgString+";}").program();
        LocalVariableDeclaration ld
	    =(LocalVariableDeclaration)((StatementBlock)pe).getStatementAt(0);

	return ld.getVariableSpecifications().
	    getVariableSpecification(0).getInitializer();

    }


    public static Proof parseTermAsInitialProof(String termstr) {
	ConstrainedFormula cf=new ConstrainedFormula(parseTerm(termstr));
	Semisequent succ=Semisequent.EMPTY_SEMISEQUENT.insert(0,cf).semisequent();
	Sequent s=Sequent.createSuccSequent(succ);
	Proof proof = new Proof(services);
	proof.setRoot(new Node(proof, s));
	return proof;
    }

}
