// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2005 Universitaet Karlsruhe, Germany
//                         Universitaet Koblenz-Landau, Germany
//                         Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
//
package de.uka.ilkd.key.rule.metaconstruct;

import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.java.expression.literal.BooleanLiteral;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.TermFactory;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.rule.inst.SVInstantiations;

public class StaticAnalysis extends AbstractMetaOperator {


    public StaticAnalysis() {
	super(new Name("#staticanalysis"), 1);
    }


    /**
     * checks whether the top level structure of the given @link Term
     * is syntactically valid, given the assumption that the top level
     * operator of the term is the same as this Operator. The
     * assumption that the top level operator and the term are equal
     * is NOT checked.  
     * @return true iff the top level structure of
     * the @link Term is valid.
     */
    public boolean validTopLevel(Term term) {
	// a meta operator accepts almost everything
	return  term.arity()==arity();
    }

    private Boolean calcInstanceof(Term term, SVInstantiations svInst, 
				   Services services) {
	Term instOfTerm = term.sub(0);
	Sort depSort = ((InstanceofSymbol)instOfTerm.op())
	    .getSortDependingOn();		
	Sort objSort = instOfTerm.sub(0).sort();
	if (objSort.extendsTrans(depSort)){
	    return new Boolean(true);
	} else if(!depSort.extendsTrans(objSort) &&
		  !objSort.extendsTrans(depSort)){
	    return new Boolean(false);
	} else{
	    return null;
	}
    }


    /** calculates the resulting term. */
    public Term calculate(Term term, SVInstantiations svInst, Services services) {
	Operator trueLitOp = services.getTypeConverter()
	    .convertToLogicElement(BooleanLiteral.TRUE).op();
	if (term.sub(0).op() instanceof Equality &&
	    term.sub(0).sub(0).op() instanceof InstanceofSymbol &&
	    term.sub(0).sub(1).op()==trueLitOp) {

	    Boolean res = calcInstanceof(term.sub(0), svInst, services);
	    if  (res==null){ // no decision
		return term.sub(0);
	    } else {
		return res.booleanValue() ? 
		    TermFactory.DEFAULT.createJunctorTerm(Op.TRUE) :
		    TermFactory.DEFAULT.createJunctorTerm(Op.FALSE);
	    }	    
	}
	return null;
    }

}
